@@ -56,7 +56,7 @@ def RegisterAgent(manager, agent_path, capability):
     if not manager.default_agent:
         manager.default_agent = agent_path
     manager.agent_paths += [agent_path]
-    manager.capabilities[agent_path] = capability
+    manager.capabilities[str(agent_path)] = capability
 
 
 @dbus.service.method(AGENT_MANAGER_IFACE,
@@ -98,6 +98,64 @@ def load(mock, _parameters):
         ('RegisterProfile', 'osa{sv}', '', ''),
         ('UnregisterProfile', 'o', '', ''),
     ])
+    bluez.agent_paths = []
+    bluez.capabilities = {}
+    bluez.default_agent = None
+
+
+@dbus.service.method(ADAPTER_IFACE,
+                     in_signature='o', out_signature='')
+def RemoveDevice(adapter, path):
+    adapter.RemoveObject(path)
+
+    manager = mockobject.objects['/']
+    manager.EmitSignal(OBJECT_MANAGER_IFACE, 'InterfacesRemoved',
+                       'oas', [
+                           dbus.ObjectPath(path),
+                           [DEVICE_IFACE],
+                       ])
+
+
+@dbus.service.method(ADAPTER_IFACE,
+                     in_signature='', out_signature='')
+def StartDiscovery(adapter):
+    adapter.discovering = True
+    # NOTE: discovery filter support is minimal to mock
+    # the Discoverable discovery filter
+    if adapter.discovery_filter is not None:
+        adapter.discoverable = True
+    adapter.EmitSignal(dbus.PROPERTIES_IFACE, 'PropertiesChanged', 'sa{sv}as', [
+        ADAPTER_IFACE,
+        {
+            'Discoverable': dbus.Boolean(adapter.discoverable, variant_level=1),
+            'Discovering': dbus.Boolean(adapter.discovering, variant_level=1),
+        },
+        [],
+    ])
+
+
+@dbus.service.method(ADAPTER_IFACE,
+                     in_signature='', out_signature='')
+def StopDiscovery(adapter):
+    adapter.discovering = False
+    # NOTE: discovery filter support is minimal to mock
+    # the Discoverable discovery filter
+    if adapter.discovery_filter is not None:
+        adapter.discoverable = False
+    adapter.EmitSignal(dbus.PROPERTIES_IFACE, 'PropertiesChanged', 'sa{sv}as', [
+        ADAPTER_IFACE,
+        {
+            'Discoverable': dbus.Boolean(adapter.discoverable, variant_level=1),
+            'Discovering': dbus.Boolean(adapter.discovering, variant_level=1),
+        },
+        [],
+    ])
+
+
+@dbus.service.method(ADAPTER_IFACE,
+                     in_signature='a{sv}', out_signature='')
+def SetDiscoveryFilter(adapter, discovery_filter):
+    adapter.discovery_filter = discovery_filter
 
 
 @dbus.service.method(BLUEZ_MOCK_IFACE,
@@ -111,6 +169,9 @@ def AddAdapter(self, device_name, system_name):
     Returns the new object path.
     '''
     path = '/org/bluez/' + device_name
+    address_start = int(device_name[-1])
+    address = f"{address_start:02d}:{address_start+1:02d}:{address_start+2:02d}:" + \
+        f"{address_start+3:02d}:{address_start+4:02d}:{address_start+5:02d}"
     adapter_properties = {
         'UUIDs': dbus.Array([
             # Reference:
@@ -126,11 +187,12 @@ def AddAdapter(self, device_name, system_name):
             # Audio/Video Remote Control Profile (target)
             '0000110c-0000-1000-8000-00805f9b34fb',
         ], variant_level=1),
-        'Discoverable': dbus.Boolean(True, variant_level=1),
-        'Discovering': dbus.Boolean(True, variant_level=1),
+        'Discoverable': dbus.Boolean(False, variant_level=1),
+        'Discovering': dbus.Boolean(False, variant_level=1),
         'Pairable': dbus.Boolean(True, variant_level=1),
         'Powered': dbus.Boolean(True, variant_level=1),
-        'Address': dbus.String('00:01:02:03:04:05', variant_level=1),
+        'Address': dbus.String(address, variant_level=1),
+        'AddressType': dbus.String('public', variant_level=1),
         'Alias': dbus.String(system_name, variant_level=1),
         'Modalias': dbus.String('usb:v1D6Bp0245d050A', variant_level=1),
         'Name': dbus.String(system_name, variant_level=1),
@@ -139,7 +201,7 @@ def AddAdapter(self, device_name, system_name):
         # bluetooth_class_of_device-service_generator.html
         'Class': dbus.UInt32(268, variant_level=1),  # Computer, Laptop
         'DiscoverableTimeout': dbus.UInt32(180, variant_level=1),
-        'PairableTimeout': dbus.UInt32(180, variant_level=1),
+        'PairableTimeout': dbus.UInt32(0, variant_level=1),
     }
 
     self.AddObject(path,
@@ -148,12 +210,16 @@ def AddAdapter(self, device_name, system_name):
                    adapter_properties,
                    # Methods
                    [
-                       ('RemoveDevice', 'o', '', ''),
-                       ('StartDiscovery', '', '', ''),
-                       ('StopDiscovery', '', '', ''),
+                       ('RemoveDevice', 'o', '', RemoveDevice),
+                       ('StartDiscovery', '', '', StartDiscovery),
+                       ('StopDiscovery', '', '', StopDiscovery),
+                       ('SetDiscoveryFilter', 'a{sv}', '', SetDiscoveryFilter),
                    ])
 
     adapter = mockobject.objects[path]
+    adapter.discovering = False
+    adapter.discoverable = False
+    adapter.discovery_filter = None
     adapter.AddMethods(MEDIA_IFACE, [
         ('RegisterEndpoint', 'oa{sv}', '', ''),
         ('UnregisterEndpoint', 'o', '', ''),
@@ -173,6 +239,26 @@ def AddAdapter(self, device_name, system_name):
     return path
 
 
+@dbus.service.method(BLUEZ_MOCK_IFACE,
+                     in_signature='s')
+def RemoveAdapter(self, device_name):
+    '''Convenience method to remove a Bluetooth adapter
+    '''
+    path = '/org/bluez/' + device_name
+    # We could remove the devices related to the adapters here, but
+    # when bluez crashes, the InterfacesRemoved aren't necessarily sent
+    # devices first, so in effect, our laziness is testing an edge case
+    # in the clients
+    self.RemoveObject(path)
+
+    manager = mockobject.objects['/']
+    manager.EmitSignal(OBJECT_MANAGER_IFACE, 'InterfacesRemoved',
+                       'oas', [
+                           dbus.ObjectPath(path),
+                           [ADAPTER_IFACE],
+                       ])
+
+
 @dbus.service.method(DEVICE_IFACE,
                      in_signature='', out_signature='')
 def Pair(device):
@@ -185,6 +271,40 @@ def Pair(device):
     device.PairDevice(adapter_device_name, device_address)
 
 
+@dbus.service.method(DEVICE_IFACE,
+                     in_signature='', out_signature='')
+def Connect(device):
+    if device.connected:
+        raise dbus.exceptions.DBusException(
+            'Already Connected',
+            name='org.bluez.Error.AlreadyConnected')
+    device.connected = True
+    device.EmitSignal(dbus.PROPERTIES_IFACE, 'PropertiesChanged', 'sa{sv}as', [
+        DEVICE_IFACE,
+        {
+            'Connected': dbus.Boolean(device.connected, variant_level=1),
+        },
+        [],
+    ])
+
+
+@dbus.service.method(DEVICE_IFACE,
+                     in_signature='', out_signature='')
+def Disconnect(device):
+    if not device.connected:
+        raise dbus.exceptions.DBusException(
+            'Not Connected',
+            name='org.bluez.Error.NotConnected')
+    device.connected = False
+    device.EmitSignal(dbus.PROPERTIES_IFACE, 'PropertiesChanged', 'sa{sv}as', [
+        DEVICE_IFACE,
+        {
+            'Connected': dbus.Boolean(device.connected, variant_level=1),
+        },
+        [],
+    ])
+
+
 @dbus.service.method(BLUEZ_MOCK_IFACE,
                      in_signature='sss', out_signature='s')
 def AddDevice(self, adapter_device_name, device_address, alias):
@@ -229,14 +349,15 @@ def AddDevice(self, adapter_device_name, device_address, alias):
                    # Methods
                    [
                        ('CancelPairing', '', '', ''),
-                       ('Connect', '', '', ''),
+                       ('Connect', '', '', Connect),
                        ('ConnectProfile', 's', '', ''),
-                       ('Disconnect', '', '', ''),
+                       ('Disconnect', '', '', Disconnect),
                        ('DisconnectProfile', 's', '', ''),
                        ('Pair', '', '', Pair),
                    ])
     device = mockobject.objects[path]
     device.paired = False
+    device.connected = False
 
     manager = mockobject.objects['/']
     manager.EmitSignal(OBJECT_MANAGER_IFACE, 'InterfacesAdded',
